// bsprite.cpp:  base sprite class functions

// updated by Lucas Keyes
// January 17, 2002

#include <string.h>  // for memcpy
#include "defines.h"  // contains declared constants
#include "bsprite.h"  // header file
#include "dxmanager.h"

extern DirectXManagerObject DXManager;  // global DirectX Manager Object

    SpriteImg_Object::SpriteImg_Object(int frames, int width, int height) {
        // settings
        s_width = width; s_height = height;  // width and height
        s_frameCount = frames;               // assign number of frames

        // create space for sprite surface pointers
        s_lpDDImage = new LPDIRECTDRAWSURFACE[frames];
        for (int i = 0; i < frames; i++) s_lpDDImage[i] = NULL;
        
        // create surfaces for sprite images
        CreateSurfaces();
    }

	SpriteImg_Object::~SpriteImg_Object() { // deconstructor
	//deallocate memory allocated in constructor
		 delete[]s_lpDDImage;
	}

    void SpriteImg_Object::CreateSurfaces() {  // create surfaces

        // set parameters of surface to be created
        DDSURFACEDESC ddsd;    // direct draw surface descriptor

        ddsd.dwSize = sizeof(ddsd);    // required field
        ddsd.dwFlags = DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH;    // attributes
        ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;        // offscreen
        ddsd.dwHeight = s_height;    // sprite height
        ddsd.dwWidth = s_width;      // sprite width

        // color key for transparent color
        DDCOLORKEY ddck;    // direct draw color descriptor
    
        ddck.dwColorSpaceLowValue = 
             ddck.dwColorSpaceHighValue = TRANSPARENT_COLOR;  // one color

        // create surfaces
        for (int i = 0; i < s_frameCount; i++) { // for each frame
            // create surface
            if (FAILED(DXManager.lpdirectdrawobj->
                          CreateSurface(&ddsd,&(s_lpDDImage[i]),NULL))) 
               s_lpDDImage[i] = NULL;

            // set the transparent color
            s_lpDDImage[i]->SetColorKey(DDCKEY_SRCBLT,&ddck);
        }
    } // end CreateSurfaces

    
    void SpriteImg_Object::Release() {  // release all sprite surfaces

        for (int i = 0; i < s_frameCount; i++) // for each frame
            if (s_lpDDImage[i] != NULL)  // if it is really there
                s_lpDDImage[i]->Release();    // release it
    }  // end Release

    BOOL SpriteImg_Object::load(SpriteBmp_Object *buffer, int frame, int x, int y) {

        // grab sprite image from (x,y) in *buffer
        // and store in s_lpDDImage[frame]

        return buffer->draw(s_lpDDImage[frame], s_width, s_height, x, y );
    }  // end load

    void SpriteImg_Object::draw(int frame, int x, int y, LPDIRECTDRAWSURFACE dest) {

        dest->BltFast(x, y, s_lpDDImage[frame], NULL, DDBLTFAST_SRCCOLORKEY|DDBLTFAST_WAIT);
    }

    BOOL SpriteImg_Object::Restore() {  // restore surfaces
        BOOL result = TRUE;    // return TRUE if all surfaces restored OK
        for (int i = 0; i < s_frameCount; i++) // for each frame
            if (s_lpDDImage[i])    // if it exists
               result = result&&SUCCEEDED(s_lpDDImage[i]->Restore());  // restore it
        return result;
    }

    // reader functions

    int SpriteImg_Object::frame_count(){  // return number of frames
        return s_frameCount;
    }

    int SpriteImg_Object::height(){  // return height
        return s_height;
    }

    int SpriteImg_Object::width(){   // return width
        return s_width;
    }


	BOOL SpriteImg_Object::load(char *filename,int x, int y,
		                        LPDIRECTDRAWPALETTE palette1,
								LPDIRECTDRAWPALETTE palette2) {

		SpriteBmp_Object *tmpbmp = new SpriteBmp_Object();

		tmpbmp->load(filename);
		tmpbmp->setpalette(palette1);
		tmpbmp->setpalette(palette2);

		load(tmpbmp,0,x,y);

		delete tmpbmp;

		return true;


	}
        