// Lucas Keyes
// Super Happy Quadblocks
// Your average, run-of-the-mill tetris clone.
// Except Happier.

// INCLUDES ///////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN 
#define DIRECTINPUT_VERSION 0x0700 
#define INITGUID				// initializes the globally unique identifiers


#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
//#include <conio.h>
//#include <stdlib.h>
//#include <malloc.h>
//#include <memory.h>
//#include <string.h>
//#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <stdlib.h>
#include <OBJBASE.H>

#include <ddraw.h>  // directX includes
#include <dinput.h>
#include <dsound.h>

// custom includes
#include "defines.h"
#include "program_methods.h"
#include "quadblock.h"
#include "soundobject.h"
#include "sbmp.h"
#include "timer.h"
#include "bsprite.h"
#include "csprite.h"

#include "dxmanager.h"
#include "SpriteImg_ManagerObject.h"
#include "mapgrid_obj.h"
#include "snd_managerobj.h"

// TYPES //////////////////////////////////////////////////

typedef unsigned short USHORT;
typedef unsigned short WORD;
typedef unsigned char  UCHAR;
typedef unsigned char  BYTE;


// PROTOTYPES /////////////////////////////////////////////

int game_Init(int[]);
int game_Main(QuadBlock &, int[], QuadBlock, int[], int *);
int game_Shutdown(int[]);

// GLOBAL VARIABLES ///////////////////////////////////////

HWND main_window_handle = NULL;
HINSTANCE main_instance_handle = NULL;

QuadBlock quadarray[8];	// holds the array of colored blocks

int total_lines = 0;	// keeps a tab on the total number of lines
						// cleared by the player.

int inputswitch = 0;	// slows down the rate at which the player
						// can move left or right

DWORD next_down_time;	// holds the time to control the falling rate of the blocks
DWORD next_rotate_time; // holds the time to control the spinning rate of the blocks

int numlife = 3;

Sound_ManagerObject *sounds_bag = NULL;

SpriteBmp_Object *quadblock_bmp = NULL;

CTimer Timer;

SpriteImg_ManagerObject *sprite_bag = NULL;

DirectXManagerObject DXManager;

mapGrid_Object mapGridArray;


// Event Handler Function
LRESULT CALLBACK WinProc(HWND hwnd,UINT msg, WPARAM wparam,
						 LPARAM lparam) {
	HDC hdc; // device context for paint message
	PAINTSTRUCT ps; // paintstructure for paint message

	switch(msg) {

		case WM_CREATE: {
			return(0);
		} break;

		case WM_PAINT: {
			hdc = BeginPaint(hwnd,&ps);
			EndPaint(hwnd,&ps);
			return(0);
					   }break;
		case WM_DESTROY: {
			PostQuitMessage(0);
			return(0);
						 } break;
		default: break;
	}
	return(DefWindowProc(hwnd,msg,wparam,lparam));
} // end WinProc



// main program function
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance,
				   LPSTR lpcmdline, int ncmdshow) {

	// variables
	WNDCLASS winclass; // main windows class
	HWND hwnd;  // generic window handle
	MSG msg;  // generic message for main loop

	
	QuadBlock nextblock, topblock;

	int filledrows[MAXMAPY];
	int numfilled = 0;
	int score[MAXSCOREPOS];
	int temp = 0;

	// fill in the window class structure
	winclass.style = CS_DBLCLKS|CS_OWNDC|CS_HREDRAW|CS_VREDRAW;
	winclass.lpfnWndProc = WinProc;
	winclass.cbClsExtra = 0;
	winclass.cbWndExtra = 0;
	winclass.hInstance = hinstance; // from WinMain params
	winclass.hIcon = LoadIcon(NULL,IDI_APPLICATION);
	winclass.hCursor = LoadCursor(NULL,IDC_ARROW);
	winclass.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName = NULL;
	winclass.lpszClassName = WINCLASSNAME; // defined as constant

	// register the class
	if (RegisterClass(&winclass) == 0)
		return(0); // error

	// create the window
	if (!(hwnd = CreateWindow(WINCLASSNAME,"Test Window",
							  WS_VISIBLE|WS_POPUP,
								0,0,
								GetSystemMetrics(SM_CXSCREEN),
								GetSystemMetrics(SM_CYSCREEN),
								NULL, NULL, hinstance,NULL)))
		return(0); // in case of error


	main_window_handle = hwnd;  // sets window handle to a global
	main_instance_handle = hinstance;

	srand(GetTickCount());		// initializes the randomizer for
								// determining the blocks
	ShowCursor(false);
	
	if(!(game_Init(score)))		// initializes the total game
		return(0);
	do {						// while loop continues until lives are
								// depleted or user chooses to quit


	setBlocks(topblock,nextblock);	// set both blocks to random and 
											// place in grid


	while(1) {

		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			if (msg.message == WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		// run through main loop, checking for ending of life or ending of the game
		temp = game_Main(topblock,score,nextblock,filledrows,&numfilled);
		
		// if ending of game or life is returned, break out of loop
		if ((temp == END_LIFE) || (temp == END_GAME))
			break;
	}

	numlife--;	// decrement the number of lives
}while ((temp != END_GAME) && (numlife > 0));	// repeat until game is ended by
												// player or player runs out of lives
	
	game_Shutdown(score);	// shutdown game and release all variables
	return(msg.wParam);
} // end WinMain


int game_Init(int score[MAXSCOREPOS]) {
	// this function initializes the fundamental elements of the
	// game, initializing directX functions and global variables

	
	sprite_bag = new SpriteImg_ManagerObject();
	sounds_bag = new Sound_ManagerObject();

	// setup DirectX
	if (!DXManager.startupDirectX(main_instance_handle,
		                     main_window_handle))
		return false;	

	// initialize all timers to the current time
	next_down_time = current_time();
	next_rotate_time = current_time();
	
	dostartScreen();

	// draw loading screen while player waits
	drawloadScreen();

	if (!sounds_bag->load(MUSIC,"techno.wav"))
		return 0;
	
	if (!sounds_bag->load(CRUNCH,"crunch.wav"))
		return 0;

	// load in menu object
	SpriteBmp_Object *tmpbmp = new SpriteBmp_Object();

	tmpbmp->load("menu1.bmp");
	tmpbmp->setpalette(DXManager.lpddsprimarypalette);
	tmpbmp->setpalette(DXManager.lpddsecondarypalette);

	sprite_bag->create(MENU,1,640,480,true);
	sprite_bag->load(MENU,tmpbmp,0,0,0);
	
	delete tmpbmp;	
	
	// load in block bitmaps
	loadBlocks();
	// load in number bitmaps
	loadScore();

	// initialize score array to all zeroes
	for (int i = 0; i < MAXSCOREPOS; i++)
		score[i] = 0;
	
	// create all QuadBlock objects (a-f)
	createQuadBlocks();	
	
	return(1);

}

int game_Main(QuadBlock &topblock,int score[MAXSCOREPOS],QuadBlock nextblock, int filledrows[],int * numfilled) {

	int temp;

	sounds_bag->play(MUSIC,true);	// start playing game music

	nextblock.copyQuadBlocks(quadarray[rand() % 7]);

	temp = processOneQuadBlock(topblock,score,nextblock);
	
	if (temp == END_LIFE)
		return END_LIFE;
	else if (temp == END_GAME)
		return END_GAME; 

	  
	if(mapGridArray.checkfilledRows(filledrows,numfilled)) {	// if filled rows
		
		addScore(score,*numfilled);							// increment score
		deleteRows(*numfilled,filledrows);			// delete filled rows
		numfilled = 0; 
	} 
	
	topblock.copyQuadBlocks(nextblock);

	return(2);
}

int game_Shutdown(int score[MAXSCOREPOS]) {

	deletesounds();

	doendScreen(score);

	DXManager.clearsecsurface();	

	// destroy all game objects
	delete sprite_bag;

	// shutdown all DirectX variables
	DXManager.shutdownDirectX();

	return(1);

}